/*-----------------------------------------------------------------------/
 * Header file for mGet: multithreaded wget
 * contains important structures for the program 
 * and included files 
 */

//------------ Includes ----------------//

#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <sys/time.h>
#include <math.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <netdb.h>
#include <string.h>
#include <arpa/inet.h>
#include <malloc.h>
#include <pthread.h>

//in the SunOS I tested, some of the strings functions were given in strings.h
//a more rigourous and proper check is needed. will find out later
#if defined(STRINGS_H)
#include <strings.h>
#endif

#if defined(HAVE_GETOPT_LONG) || defined(HAVE_GETOPT_GETOPT_H)
#include <getopt.h>
#elif defined(HAVE_GETOPT_STDLIB_H)
#include <stdlib.h>
#else
error "getopt or getopt_long is not supported"
#endif

#if defined (HAVE_GETOPT_LONG)
# define no_argument            0
# define required_argument      1
# define optional_argument      2
#endif

//------------- Typedefs ------------------//
typedef struct sockaddr SADDR;
typedef struct sockaddr_in SADDR_IN;
typedef struct in_addr INADDR;
typedef struct hostent HOST;
typedef unsigned char byte;

//------------- Platform specific stuff-----//
#ifndef __WIN32 //initially done with a view of porting, not carried forward
typedef int SOCKET;
#define closesocket(x) close(x)
#endif

/* herror() is a cool function: but not found in Sun */
#ifndef HAS_HERROR
#define herror(A) printf("%s\n",A)
#endif

#if !defined(HAVE_INET_NTOP)
char *inet_ntop(int , INADDR *, char *, size_t );
#endif

#ifndef NULL 
#define NULL 0 //some compilers complained
#endif

#ifndef INET_ADDRSTRLEN
#define INET_ADDRSTRLEN 33
#endif

//------------- Constants -----------------//
#define VERSION "1.3.2.2"
#define LENGTH 1024
#define MAX_SEGMENT 10
#define CLOSE_SOCK 1
#define NOCLOSE_SOCK 0
#define PORT 80 //default port
#define PROXY_PORT 3128	
#define min(a,b) ((a>=b)?b:a)
#define max(a,b) ((a>=b)?a:b)
#define strlen(x) ((x==NULL)?0:strlen(x)) //avoid segmentation fault
#define GET_STR \
"GET %s://%s HTTP/1.1 \r\nHOST:%s\r\nUser-Agent: mGet/"VERSION"\r\nPragma:no-cache\r\nCache-Control:no-cache\r\nAccept:*/*\r\nRange: bytes=%ld-%ld\r\n%s%s%s%sConnection:close\r\n\r\n"

#define GET_STR_NO_RANGE \
"GET %s://%s HTTP/1.1 \r\nHOST:%s\r\nUser-Agent: mGet/"VERSION"\r\nPragma:no-cache\r\nCache-Control:no-cache\r\nAccept:*/*\r\n%s%s%s%sConnection:close\r\n\r\n"

#define HEAD_STR_HTTP \
"HEAD %s://%s HTTP/1.1 \r\nHOST:%s\r\nUser-Agent: mGet/"VERSION"\r\nPragma:no-cache\r\nCache-Control:no-cache\r\nAccept:*/*\r\nRange: bytes=1-2\r\n%s%s%s%sConnection:close\r\n\r\n" 

#define HEAD_STR_FTP \
"GET %s://%s HTTP/1.1 \r\nHOST:%s\r\nUser-Agent: mGet/"VERSION"\r\nPragma:no-cache\r\nCache-Control:no-cache\r\nAccept:*/*\r\nRange: bytes=1-2\r\n%s%s%s%sConnection:close\r\n\r\n" 

#define PROXY_AUTH \
"Proxy-Authorization:Basic "

#define REFERRER_DECL \
"Referer: "

#ifndef __MGET_H
#define __MGET_H

#include "mgetutil.h"

//------------- Global Variables ----------//
int num_segment=1; //number of segment the file can be broken into, default 1
int port=PORT, proxy_port=PROXY_PORT; 
char *proxy=NULL;
char *host_name=NULL;
char *file_name=NULL; //name of the file to get
char *save_file_name=NULL; //name of the file to save
char *user_name=NULL, *user_passwd=NULL;
pthread_t *thread_id; //pointer to store thread ids
FILE *file, *outfile=NULL; //default output to stdout
int thread_timeout_second=60;
int RETRY_TIME=5; //retry for socket create/connect error
char protocol[5]="http"; //default
char *referrer=NULL;
int rollback=0;  //rollback, default=0 ;DO U REALLY NEED TO ROLLBACK

//parameters
typedef enum bool {TRUE, FALSE} BOOL;
BOOL PROXY=FALSE; //default ... no proxy
BOOL REFERRER=FALSE; //default no referrer
BOOL SUPPORT_RESUME=TRUE; //default, support resume
BOOL HAS_LENGTH=TRUE; //default file length is returned
BOOL PROTO_HTTP; //default protocol: HTTP

int curr_threads=0;
pthread_mutex_t thread_done_mutex;
pthread_cond_t thread_done_cond;
pthread_mutex_t file_mutex;

SOCKET s; //primary socket
SADDR_IN saddr;
HOST *host;

//------------- Structures ----------------//
struct request{
	long size_file;
	char *version; //HTTP version of server, actually 9 fields
	int status_code; //status-code of file
}; //contains the informations for a request
typedef struct request REQUEST;

struct present_page{
	unsigned int page_id; //maybe needed to identify a page
	long size_total; //total size: reduntant
	long range_start; //start byte
	long range_end; //end byte
	long range_size; // range size: reduntant
	SOCKET sock; //thread socket
	long present_seek_pos; //present lseek position
	long present_read_pos; //present position till which file is read
}; //contains the information for the present page
typedef struct present_page PPAGE;

//error codes
#define ERR_M_OK 0
#define ERR_M_SOCKERR -1 //socket create error
#define ERR_M_CONNERR -2 //connect error
#define ERR_M_TIMEOUT -3 //timeout

//------------- Function Declarations ------//
void print_usage(); //prints usage information and exits
void get_options(int , char **); //get the options
void err_exit(char *, int, SOCKET); //print error,process by option and exit
void init_socket(); //initialize the main sock
int get_head(REQUEST *); //get the header for this request
void get_name_passwd(void); //get the user name and password
void process_thread(REQUEST *); //start the threads for the request
void* start_thread(void *); //function on each thread
int start_thread_sock(PPAGE *); //thread socket function
int get_thread_request(PPAGE *); //thread specific download routine
void print_segments(); //to print individual segment status and values
int check_file_segment_num(int, int); //checks if the number of segment is not too large

#endif

REQUEST main_request;
PPAGE *ppage;
char *sep="+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n";//separator string
char *info="
        ------------ mGet - "VERSION"--------------\n
	----------------- Made by -----------------\n
	-------- Arindam Chakraborty 98071 --------\n
	-------- Ashutosh Kumar      98091 --------\n
	-------- Chinmoy Dutta       98115 --------\n
	-------- Debajyoti Bera      98117 --------\n
	----- as a part of Assignment #1, CS425 ---\n";	

//----------- end of mget.h --------------//	
